/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * Portions Copyrighted 2010 Martin Rexa
 */

package cz.xelfi.quoridor.statistics;

import cz.xelfi.quoridor.webidor.Game;
import cz.xelfi.quoridor.webidor.GameId;
import cz.xelfi.quoridor.webidor.GameStatus;
import cz.xelfi.quoridor.webidor.User;
import cz.xelfi.quoridor.Move;
import java.util.HashMap;
import java.util.Map;
import java.util.List;
import java.util.ArrayList;
import javax.xml.bind.annotation.XmlAttribute;

/**
 *
 * @author Martin Rexa
 */
public class OpeningTreeNode {
    @XmlAttribute
    String hashCode;
    List<GameId> gameIds;
    List<GameId> finishedGames;
    Map<Move, OpeningTreeNode> children;

    public OpeningTreeNode(){
    }

    public OpeningTreeNode(String hashCode){
        this.hashCode = hashCode;
        gameIds = new ArrayList<GameId>();
        finishedGames = new ArrayList<GameId>();
        children = new HashMap<Move,OpeningTreeNode>();
    }

    public OpeningTreeNode addGame(Game game){
        gameIds.add(game.getId());
        return this;
    }

    public OpeningTreeNode addFinishedGame(Game game){
        finishedGames.add(game.getId());
        return this;
    }

    public OpeningTreeNode addGameId(GameId gameId){
        gameIds.add(gameId);
        return this;
    }

    public OpeningTreeNode addChild(Move move, OpeningTreeNode child){
        children.put(move, child);
        return this;
    }

    public OpeningNodeView getView(User user){
        return getView(user.getId());
    }

    public OpeningNodeView getView(String userId){
        OpeningNodeView view = new OpeningNodeView(hashCode);
        for(Map.Entry<Move, OpeningTreeNode> e: children.entrySet()){
            for(GameId gId: e.getValue().gameIds){
                if(User.canSee(gId, userId)){
                    view.processGame(e.getKey(), e.getValue().hashCode, gId);
                }
            }
        }
        for(GameId gId: finishedGames){
            if(User.canSee(gId, userId)){
                view.addFinishedGame(gId);
            }
        }
        return view;
    }

    public List<GameId> filterGames(String userId, GameStatus status){
        List<GameId> result = new ArrayList<GameId>();
        for(GameId gId: gameIds){
            if(User.canSee(gId, userId))
                if(gId.getStatus().equals(status))
                    result.add(gId);
        }
        return result;
    }

    public List<GameId> getGameIds(){
        return gameIds;
    }

//    @XmlAttribute
//    @XmlElement
    public String getHashCode(){
        return this.hashCode;
    }


    @Override
    public boolean equals(Object obj) {
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        OpeningTreeNode other = (OpeningTreeNode)obj;
        return hashCode.equals(other.hashCode);
    }

    @Override
    public int hashCode() {
        return hashCode.hashCode();
    }

}
